@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
 
@@ -18,4 +19,5 @@ from .segmentation import (Atropos, LaplacianThickness, N4BiasFieldCorrection, J
 from .visualization import ConvertScalarImageToRGB, CreateTiledMosaic
 
 # Utility Programs
-from .utils import AverageAffineTransform, AverageImages, MultiplyImages, JacobianDeterminant
+from .utils import (AverageAffineTransform, AverageImages, MultiplyImages,
+                    CreateJacobianDeterminantImage, AffineInitializer)
