@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """The ants module provides basic functions for interfacing with ants functions.
 
    Change directory to provide relative paths for doctests
@@ -6,21 +7,19 @@
    >>> datadir = os.path.realpath(os.path.join(filepath, '../../testing/data'))
    >>> os.chdir(datadir)
 """
-
-from builtins import range
-
-from ..base import TraitedSpec, File, traits, InputMultiPath
-from .base import ANTSCommand, ANTSCommandInputSpec
+from __future__ import print_function, division, unicode_literals, absolute_import
+from builtins import range, str
 import os
-from ..traits_extension import isdefined
+
+from ..base import TraitedSpec, File, Str, traits, InputMultiPath, isdefined
+from .base import ANTSCommand, ANTSCommandInputSpec
 
 
 class ANTSInputSpec(ANTSCommandInputSpec):
     dimension = traits.Enum(3, 2, argstr='%d', usedefault=False,
                             position=1, desc='image dimension (2 or 3)')
     fixed_image = InputMultiPath(File(exists=True), mandatory=True,
-                                 desc=('image to apply transformation to (generally a coregistered '
-                                       'functional)'))
+                                 desc=('image to which the moving image is warped'))
     moving_image = InputMultiPath(File(exists=True), argstr='%s',
                                   mandatory=True,
                                   desc=('image to apply transformation to (generally a coregistered '
@@ -52,7 +51,7 @@ class ANTSInputSpec(ANTSCommandInputSpec):
     metric_weight = traits.List(traits.Float(), requires=['metric'], desc='')
     radius = traits.List(traits.Int(), requires=['metric'], desc='')
 
-    output_transform_prefix = traits.Str('out', usedefault=True,
+    output_transform_prefix = Str('out', usedefault=True,
                                          argstr='--output-naming %s',
                                          mandatory=True, desc='')
     transformation_model = traits.Enum('Diff', 'Elast', 'Exp', 'Greedy Exp',
@@ -120,7 +119,7 @@ class ANTS(ANTSCommand):
     >>> ants.inputs.regularization_gradient_field_sigma = 3
     >>> ants.inputs.regularization_deformation_field_sigma = 0
     >>> ants.inputs.number_of_affine_iterations = [10000,10000,10000,10000,10000]
-    >>> ants.cmdline
+    >>> ants.cmdline # doctest: +ALLOW_UNICODE
     'ANTS 3 --MI-option 32x16000 --image-metric CC[ T1.nii, resting.nii, 1, 5 ] --number-of-affine-iterations \
 10000x10000x10000x10000x10000 --number-of-iterations 50x35x15 --output-naming MY --regularization Gauss[3.0,0.0] \
 --transformation-model SyN[0.25] --use-Histogram-Matching 1'
@@ -235,7 +234,7 @@ class RegistrationInputSpec(ANTSCommandInputSpec):
     initial_moving_transform = File(argstr='%s', exists=True, desc='',
                                     xor=['initial_moving_transform_com'])
     invert_initial_moving_transform = traits.Bool(requires=["initial_moving_transform"],
-        desc='', xor=['initial_moving_transform_com'])
+                                                  desc='', xor=['initial_moving_transform_com'])
 
     initial_moving_transform_com = traits.Enum(0, 1, 2, argstr='%s',
                                                default=0, xor=['initial_moving_transform'],
@@ -367,7 +366,7 @@ class RegistrationInputSpec(ANTSCommandInputSpec):
     convergence_window_size = traits.List(trait=traits.Int(), value=[
                                           10], minlen=1, requires=['convergence_threshold'], usedefault=True)
     # Output flags
-    output_transform_prefix = traits.Str(
+    output_transform_prefix = Str(
         "transform", usedefault=True, argstr="%s", desc="")
     output_warped_image = traits.Either(
         traits.Bool, File(), hash_files=False, desc="")
@@ -379,6 +378,8 @@ class RegistrationInputSpec(ANTSCommandInputSpec):
     winsorize_lower_quantile = traits.Range(
         low=0.0, high=1.0, value=0.0, argstr='%s', usedefault=True, desc="The Lower quantile to clip image ranges")
 
+    verbose = traits.Bool(argstr='-v', default=False)
+
 
 class RegistrationOutputSpec(TraitedSpec):
     forward_transforms = traits.List(
@@ -408,7 +409,6 @@ class Registration(ANTSCommand):
     >>> reg.inputs.moving_image = 'moving1.nii'
     >>> reg.inputs.output_transform_prefix = "output_"
     >>> reg.inputs.initial_moving_transform = 'trans.mat'
-    >>> reg.inputs.invert_initial_moving_transform = True
     >>> reg.inputs.transforms = ['Affine', 'SyN']
     >>> reg.inputs.transform_parameters = [(2.0,), (0.25, 3.0, 0.0)]
     >>> reg.inputs.number_of_iterations = [[1500, 200], [100, 50, 30]]
@@ -429,10 +429,21 @@ class Registration(ANTSCommand):
     >>> reg.inputs.use_estimate_learning_rate_once = [True, True]
     >>> reg.inputs.use_histogram_matching = [True, True] # This is the default
     >>> reg.inputs.output_warped_image = 'output_warped_image.nii.gz'
+    >>> reg.cmdline # doctest: +ALLOW_UNICODE
+    'antsRegistration --collapse-output-transforms 0 --dimensionality 3 --initial-moving-transform [ trans.mat, 0 ] \
+--initialize-transforms-per-stage 0 --interpolation Linear --output [ output_, output_warped_image.nii.gz ] \
+--transform Affine[ 2.0 ] --metric Mattes[ fixed1.nii, moving1.nii, 1, 32, Random, 0.05 ] \
+--convergence [ 1500x200, 1e-08, 20 ] --smoothing-sigmas 1.0x0.0vox --shrink-factors 2x1 \
+--use-estimate-learning-rate-once 1 --use-histogram-matching 1 --transform SyN[ 0.25, 3.0, 0.0 ] \
+--metric Mattes[ fixed1.nii, moving1.nii, 1, 32 ] --convergence [ 100x50x30, 1e-09, 20 ] \
+--smoothing-sigmas 2.0x1.0x0.0vox --shrink-factors 3x2x1 --use-estimate-learning-rate-once 1 \
+--use-histogram-matching 1 --winsorize-image-intensities [ 0.0, 1.0 ]  --write-composite-transform 1'
+    >>> reg.run()  # doctest: +SKIP
 
+    >>> reg.inputs.invert_initial_moving_transform = True
     >>> reg1 = copy.deepcopy(reg)
     >>> reg1.inputs.winsorize_lower_quantile = 0.025
-    >>> reg1.cmdline
+    >>> reg1.cmdline # doctest: +ALLOW_UNICODE
     'antsRegistration --collapse-output-transforms 0 --dimensionality 3 --initial-moving-transform [ trans.mat, 1 ] \
 --initialize-transforms-per-stage 0 --interpolation Linear --output [ output_, output_warped_image.nii.gz ] \
 --transform Affine[ 2.0 ] --metric Mattes[ fixed1.nii, moving1.nii, 1, 32, Random, 0.05 ] \
@@ -445,7 +456,7 @@ class Registration(ANTSCommand):
 
     >>> reg2 = copy.deepcopy(reg)
     >>> reg2.inputs.winsorize_upper_quantile = 0.975
-    >>> reg2.cmdline
+    >>> reg2.cmdline # doctest: +ALLOW_UNICODE
     'antsRegistration --collapse-output-transforms 0 --dimensionality 3 --initial-moving-transform [ trans.mat, 1 ] \
 --initialize-transforms-per-stage 0 --interpolation Linear --output [ output_, output_warped_image.nii.gz ] \
 --transform Affine[ 2.0 ] --metric Mattes[ fixed1.nii, moving1.nii, 1, 32, Random, 0.05 ] \
@@ -458,7 +469,7 @@ class Registration(ANTSCommand):
     >>> reg3 = copy.deepcopy(reg)
     >>> reg3.inputs.winsorize_lower_quantile = 0.025
     >>> reg3.inputs.winsorize_upper_quantile = 0.975
-    >>> reg3.cmdline
+    >>> reg3.cmdline # doctest: +ALLOW_UNICODE
     'antsRegistration --collapse-output-transforms 0 --dimensionality 3 --initial-moving-transform [ trans.mat, 1 ] \
 --initialize-transforms-per-stage 0 --interpolation Linear --output [ output_, output_warped_image.nii.gz ] \
 --transform Affine[ 2.0 ] --metric Mattes[ fixed1.nii, moving1.nii, 1, 32, Random, 0.05 ] \
@@ -470,7 +481,7 @@ class Registration(ANTSCommand):
 
     >>> reg3a = copy.deepcopy(reg)
     >>> reg3a.inputs.float = True
-    >>> reg3a.cmdline
+    >>> reg3a.cmdline # doctest: +ALLOW_UNICODE
     'antsRegistration --collapse-output-transforms 0 --dimensionality 3 --float 1 \
 --initial-moving-transform [ trans.mat, 1 ] --initialize-transforms-per-stage 0 --interpolation Linear \
 --output [ output_, output_warped_image.nii.gz ] --transform Affine[ 2.0 ] \
@@ -483,7 +494,7 @@ class Registration(ANTSCommand):
 
     >>> reg3b = copy.deepcopy(reg)
     >>> reg3b.inputs.float = False
-    >>> reg3b.cmdline
+    >>> reg3b.cmdline # doctest: +ALLOW_UNICODE
     'antsRegistration --collapse-output-transforms 0 --dimensionality 3 --float 0 \
 --initial-moving-transform [ trans.mat, 1 ] --initialize-transforms-per-stage 0 --interpolation Linear \
 --output [ output_, output_warped_image.nii.gz ] --transform Affine[ 2.0 ] \
@@ -501,7 +512,7 @@ class Registration(ANTSCommand):
     >>> reg4.inputs.initialize_transforms_per_stage = True
     >>> reg4.inputs.collapse_output_transforms = True
     >>> outputs = reg4._list_outputs()
-    >>> pprint.pprint(outputs)  # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
+    >>> pprint.pprint(outputs)  # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE +ALLOW_UNICODE
     {'composite_transform': '.../nipype/testing/data/output_Composite.h5',
      'forward_invert_flags': [],
      'forward_transforms': [],
@@ -511,7 +522,7 @@ class Registration(ANTSCommand):
      'reverse_transforms': [],
      'save_state': '.../nipype/testing/data/trans.mat',
      'warped_image': '.../nipype/testing/data/output_warped_image.nii.gz'}
-    >>> reg4.cmdline
+    >>> reg4.cmdline # doctest: +ALLOW_UNICODE
     'antsRegistration --collapse-output-transforms 1 --dimensionality 3 --initial-moving-transform [ trans.mat, 1 ] \
 --initialize-transforms-per-stage 1 --interpolation Linear --output [ output_, output_warped_image.nii.gz ] \
 --restore-state trans.mat --save-state trans.mat --transform Affine[ 2.0 ] \
@@ -526,7 +537,7 @@ class Registration(ANTSCommand):
     >>> reg4b = copy.deepcopy(reg4)
     >>> reg4b.inputs.write_composite_transform = False
     >>> outputs = reg4b._list_outputs()
-    >>> pprint.pprint(outputs)  # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
+    >>> pprint.pprint(outputs)  # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE +ALLOW_UNICODE
     {'composite_transform': <undefined>,
      'forward_invert_flags': [False, False],
      'forward_transforms': ['.../nipype/testing/data/output_0GenericAffine.mat',
@@ -539,7 +550,7 @@ class Registration(ANTSCommand):
      'save_state': '.../nipype/testing/data/trans.mat',
      'warped_image': '.../nipype/testing/data/output_warped_image.nii.gz'}
     >>> reg4b.aggregate_outputs()  # doctest: +SKIP
-    >>> reg4b.cmdline
+    >>> reg4b.cmdline # doctest: +ALLOW_UNICODE
     'antsRegistration --collapse-output-transforms 1 --dimensionality 3 --initial-moving-transform [ trans.mat, 1 ] \
 --initialize-transforms-per-stage 1 --interpolation Linear --output [ output_, output_warped_image.nii.gz ] \
 --restore-state trans.mat --save-state trans.mat --transform Affine[ 2.0 ] \
@@ -559,7 +570,7 @@ class Registration(ANTSCommand):
     >>> reg5.inputs.radius_or_number_of_bins = [32, [32, 4] ]
     >>> reg5.inputs.sampling_strategy = ['Random', None] # use default strategy in second stage
     >>> reg5.inputs.sampling_percentage = [0.05, [0.05, 0.10]]
-    >>> reg5.cmdline
+    >>> reg5.cmdline # doctest: +ALLOW_UNICODE
     'antsRegistration --collapse-output-transforms 0 --dimensionality 3 --initial-moving-transform [ trans.mat, 1 ] \
 --initialize-transforms-per-stage 0 --interpolation Linear --output [ output_, output_warped_image.nii.gz ] \
 --transform Affine[ 2.0 ] --metric Mattes[ fixed1.nii, moving1.nii, 1, 32, Random, 0.05 ] \
@@ -574,7 +585,7 @@ class Registration(ANTSCommand):
     >>> reg6 = copy.deepcopy(reg5)
     >>> reg6.inputs.fixed_image = ['fixed1.nii', 'fixed2.nii']
     >>> reg6.inputs.moving_image = ['moving1.nii', 'moving2.nii']
-    >>> reg6.cmdline
+    >>> reg6.cmdline # doctest: +ALLOW_UNICODE
     'antsRegistration --collapse-output-transforms 0 --dimensionality 3 --initial-moving-transform [ trans.mat, 1 ] \
 --initialize-transforms-per-stage 0 --interpolation Linear --output [ output_, output_warped_image.nii.gz ] \
 --transform Affine[ 2.0 ] --metric Mattes[ fixed1.nii, moving1.nii, 1, 32, Random, 0.05 ] \
@@ -589,7 +600,7 @@ class Registration(ANTSCommand):
     >>> reg7a = copy.deepcopy(reg)
     >>> reg7a.inputs.interpolation = 'BSpline'
     >>> reg7a.inputs.interpolation_parameters = (3,)
-    >>> reg7a.cmdline
+    >>> reg7a.cmdline # doctest: +ALLOW_UNICODE
     'antsRegistration --collapse-output-transforms 0 --dimensionality 3 --initial-moving-transform [ trans.mat, 1 ] \
 --initialize-transforms-per-stage 0 --interpolation BSpline[ 3 ] --output [ output_, output_warped_image.nii.gz ] \
 --transform Affine[ 2.0 ] --metric Mattes[ fixed1.nii, moving1.nii, 1, 32, Random, 0.05 ] \
@@ -603,7 +614,7 @@ class Registration(ANTSCommand):
     >>> reg7b = copy.deepcopy(reg)
     >>> reg7b.inputs.interpolation = 'Gaussian'
     >>> reg7b.inputs.interpolation_parameters = (1.0, 1.0)
-    >>> reg7b.cmdline
+    >>> reg7b.cmdline # doctest: +ALLOW_UNICODE
     'antsRegistration --collapse-output-transforms 0 --dimensionality 3 --initial-moving-transform [ trans.mat, 1 ] \
 --initialize-transforms-per-stage 0 --interpolation Gaussian[ 1.0, 1.0 ] \
 --output [ output_, output_warped_image.nii.gz ] --transform Affine[ 2.0 ] \
@@ -618,7 +629,7 @@ class Registration(ANTSCommand):
     >>> reg8 = copy.deepcopy(reg)
     >>> reg8.inputs.transforms = ['Affine', 'BSplineSyN']
     >>> reg8.inputs.transform_parameters = [(2.0,), (0.25, 26, 0, 3)]
-    >>> reg8.cmdline
+    >>> reg8.cmdline # doctest: +ALLOW_UNICODE
     'antsRegistration --collapse-output-transforms 0 --dimensionality 3 --initial-moving-transform [ trans.mat, 1 ] \
 --initialize-transforms-per-stage 0 --interpolation Linear --output [ output_, output_warped_image.nii.gz ] \
 --transform Affine[ 2.0 ] --metric Mattes[ fixed1.nii, moving1.nii, 1, 32, Random, 0.05 ] \
@@ -812,17 +823,13 @@ class Registration(ANTSCommand):
         elif opt == 'transforms':
             return self._format_registration()
         elif opt == 'initial_moving_transform':
-            try:
-                do_invert_transform = int(self.inputs.invert_initial_moving_transform)
-            except ValueError:
-                do_invert_transform = 0  # Just do the default behavior
+            do_invert_transform = self.inputs.invert_initial_moving_transform \
+                if isdefined(self.inputs.invert_initial_moving_transform) else 0 # Just do the default behavior
             return '--initial-moving-transform [ %s, %d ]' % (self.inputs.initial_moving_transform,
                                                               do_invert_transform)
         elif opt == 'initial_moving_transform_com':
-            try:
-                do_center_of_mass_init = int(self.inputs.initial_moving_transform_com)
-            except ValueError:
-                do_center_of_mass_init = 0  # Just do the default behavior
+            do_center_of_mass_init = self.inputs.initial_moving_transform_com \
+                if isdefined(self.inputs.initial_moving_transform_com) else 0  # Just do the default behavior
             return '--initial-moving-transform [ %s, %s, %d ]' % (self.inputs.fixed_image[0],
                                                                   self.inputs.moving_image[0],
                                                                   do_center_of_mass_init)
