@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """The ants module provides basic functions for interfacing with ants functions.
 
    Change directory to provide relative paths for doctests
@@ -7,14 +8,15 @@
    >>> os.chdir(datadir)
 
 """
+from __future__ import print_function, division, unicode_literals, absolute_import
+from builtins import range, str
 
-from builtins import range
-
-from ..base import TraitedSpec, File, traits, InputMultiPath, OutputMultiPath, isdefined
-from ...utils.filemanip import split_filename
-from .base import ANTSCommand, ANTSCommandInputSpec
 import os
-from ...utils.filemanip import copyfile
+from ...external.due import BibTeX
+from ...utils.filemanip import split_filename, copyfile
+from ..base import (TraitedSpec, File, traits, InputMultiPath, OutputMultiPath, isdefined,
+                    _exists_in_path)
+from .base import ANTSCommand, ANTSCommandInputSpec
 
 
 class AtroposInputSpec(ANTSCommandInputSpec):
@@ -89,11 +91,12 @@ class Atropos(ANTSCommand):
     >>> at.inputs.posterior_formulation = 'Socrates'
     >>> at.inputs.use_mixture_model_proportions = True
     >>> at.inputs.save_posteriors = True
-    >>> at.cmdline
+    >>> at.cmdline # doctest: +ALLOW_UNICODE
     'Atropos --image-dimensionality 3 --icm [1,1] \
 --initialization PriorProbabilityImages[2,priors/priorProbImages%02d.nii,0.8,1e-07] --intensity-image structural.nii \
 --likelihood-model Gaussian --mask-image mask.nii --mrf [0.2,1x1x1] --convergence [5,1e-06] \
 --output [structural_labeled.nii,POSTERIOR_%02d.nii.gz] --posterior-formulation Socrates[1] --use-random-seed 1'
+
     """
     input_spec = AtroposInputSpec
     output_spec = AtroposOutputSpec
@@ -206,7 +209,7 @@ class LaplacianThickness(ANTSCommand):
     >>> cort_thick.inputs.input_wm = 'white_matter.nii.gz'
     >>> cort_thick.inputs.input_gm = 'gray_matter.nii.gz'
     >>> cort_thick.inputs.output_image = 'output_thickness.nii.gz'
-    >>> cort_thick.cmdline
+    >>> cort_thick.cmdline # doctest: +ALLOW_UNICODE
     'LaplacianThickness white_matter.nii.gz gray_matter.nii.gz output_thickness.nii.gz'
 
     """
@@ -288,7 +291,7 @@ class N4BiasFieldCorrection(ANTSCommand):
     >>> n4.inputs.bspline_fitting_distance = 300
     >>> n4.inputs.shrink_factor = 3
     >>> n4.inputs.n_iterations = [50,50,30,20]
-    >>> n4.cmdline
+    >>> n4.cmdline # doctest: +ALLOW_UNICODE
     'N4BiasFieldCorrection --bspline-fitting [ 300 ] \
 -d 3 --input-image structural.nii \
 --convergence [ 50x50x30x20 ] --output structural_corrected.nii \
@@ -296,7 +299,7 @@ class N4BiasFieldCorrection(ANTSCommand):
 
     >>> n4_2 = copy.deepcopy(n4)
     >>> n4_2.inputs.convergence_threshold = 1e-6
-    >>> n4_2.cmdline
+    >>> n4_2.cmdline # doctest: +ALLOW_UNICODE
     'N4BiasFieldCorrection --bspline-fitting [ 300 ] \
 -d 3 --input-image structural.nii \
 --convergence [ 50x50x30x20, 1e-06 ] --output structural_corrected.nii \
@@ -304,7 +307,7 @@ class N4BiasFieldCorrection(ANTSCommand):
 
     >>> n4_3 = copy.deepcopy(n4_2)
     >>> n4_3.inputs.bspline_order = 5
-    >>> n4_3.cmdline
+    >>> n4_3.cmdline # doctest: +ALLOW_UNICODE
     'N4BiasFieldCorrection --bspline-fitting [ 300, 5 ] \
 -d 3 --input-image structural.nii \
 --convergence [ 50x50x30x20, 1e-06 ] --output structural_corrected.nii \
@@ -314,7 +317,7 @@ class N4BiasFieldCorrection(ANTSCommand):
     >>> n4_4.inputs.input_image = 'structural.nii'
     >>> n4_4.inputs.save_bias = True
     >>> n4_4.inputs.dimension = 3
-    >>> n4_4.cmdline
+    >>> n4_4.cmdline # doctest: +ALLOW_UNICODE
     'N4BiasFieldCorrection -d 3 --input-image structural.nii \
 --output [ structural_corrected.nii, structural_bias.nii ]'
     """
@@ -498,12 +501,15 @@ class CorticalThickness(ANTSCommand):
     >>> corticalthickness.inputs.anatomical_image ='T1.nii.gz'
     >>> corticalthickness.inputs.brain_template = 'study_template.nii.gz'
     >>> corticalthickness.inputs.brain_probability_mask ='ProbabilityMaskOfStudyTemplate.nii.gz'
-    >>> corticalthickness.inputs.segmentation_priors = ['BrainSegmentationPrior01.nii.gz', \
-    'BrainSegmentationPrior02.nii.gz', 'BrainSegmentationPrior03.nii.gz', 'BrainSegmentationPrior04.nii.gz']
+    >>> corticalthickness.inputs.segmentation_priors = ['BrainSegmentationPrior01.nii.gz',
+    ...                                                 'BrainSegmentationPrior02.nii.gz',
+    ...                                                 'BrainSegmentationPrior03.nii.gz',
+    ...                                                 'BrainSegmentationPrior04.nii.gz']
     >>> corticalthickness.inputs.t1_registration_template = 'brain_study_template.nii.gz'
-    >>> corticalthickness.cmdline
+    >>> corticalthickness.cmdline # doctest: +ALLOW_UNICODE
     'antsCorticalThickness.sh -a T1.nii.gz -m ProbabilityMaskOfStudyTemplate.nii.gz -e study_template.nii.gz -d 3 \
 -s nii.gz -o antsCT_ -p nipype_priors/BrainSegmentationPrior%02d.nii.gz -t brain_study_template.nii.gz'
+
     """
 
     input_spec = CorticalThicknessInputSpec
@@ -651,6 +657,22 @@ class BrainExtractionInputSpec(ANTSCommandInputSpec):
 class BrainExtractionOutputSpec(TraitedSpec):
     BrainExtractionMask = File(exists=True, desc='brain extraction mask')
     BrainExtractionBrain = File(exists=True, desc='brain extraction image')
+    BrainExtractionCSF = File(exists=True, desc='segmentation mask with only CSF')
+    BrainExtractionGM = File(exists=True, desc='segmentation mask with only grey matter')
+    BrainExtractionInitialAffine = File(exists=True, desc='')
+    BrainExtractionInitialAffineFixed = File(exists=True, desc='')
+    BrainExtractionInitialAffineMoving = File(exists=True, desc='')
+    BrainExtractionLaplacian = File(exists=True, desc='')
+    BrainExtractionPrior0GenericAffine = File(exists=True, desc='')
+    BrainExtractionPrior1InverseWarp = File(exists=True, desc='')
+    BrainExtractionPrior1Warp = File(exists=True, desc='')
+    BrainExtractionPriorWarped = File(exists=True, desc='')
+    BrainExtractionSegmentation = File(exists=True, desc='segmentation mask with CSF, GM, and WM')
+    BrainExtractionTemplateLaplacian = File(exists=True, desc='')
+    BrainExtractionTmp = File(exists=True, desc='')
+    BrainExtractionWM = File(exists=True, desc='segmenration mask with only white matter')
+    N4Corrected0 = File(exists=True, desc='N4 bias field corrected image')
+    N4Truncated0 = File(exists=True, desc='')
 
 
 class BrainExtraction(ANTSCommand):
@@ -663,7 +685,7 @@ class BrainExtraction(ANTSCommand):
     >>> brainextraction.inputs.anatomical_image ='T1.nii.gz'
     >>> brainextraction.inputs.brain_template = 'study_template.nii.gz'
     >>> brainextraction.inputs.brain_probability_mask ='ProbabilityMaskOfStudyTemplate.nii.gz'
-    >>> brainextraction.cmdline
+    >>> brainextraction.cmdline # doctest: +ALLOW_UNICODE
     'antsBrainExtraction.sh -a T1.nii.gz -m ProbabilityMaskOfStudyTemplate.nii.gz -e study_template.nii.gz -d 3 \
 -s nii.gz -o highres001_'
     """
@@ -671,6 +693,40 @@ class BrainExtraction(ANTSCommand):
     output_spec = BrainExtractionOutputSpec
     _cmd = 'antsBrainExtraction.sh'
 
+    def _run_interface(self, runtime, correct_return_codes=(0,)):
+        # antsBrainExtraction.sh requires ANTSPATH to be defined
+        out_environ = self._get_environ()
+        if out_environ.get('ANTSPATH') is None:
+            runtime.environ.update(out_environ)
+            executable_name = self.cmd.split()[0]
+            exist_val, cmd_path = _exists_in_path(executable_name, runtime.environ)
+            if not exist_val:
+                raise IOError("command '%s' could not be found on host %s" %
+                              (self.cmd.split()[0], runtime.hostname))
+
+            # Set the environment variable if found
+            runtime.environ.update({'ANTSPATH': os.path.dirname(cmd_path)})
+
+        runtime = super(BrainExtraction, self)._run_interface(runtime)
+
+        # Still, double-check if it didn't found N4
+        if 'we cant find' in runtime.stdout:
+            for line in runtime.stdout.split('\n'):
+                if line.strip().startswith('we cant find'):
+                    tool = line.strip().replace('we cant find the', '').split(' ')[0]
+                    break
+
+            errmsg = ('antsBrainExtraction.sh requires %s the environment variable '
+                      'ANTSPATH to be defined' % tool)
+            if runtime.stderr is None:
+                runtime.stderr = errmsg
+            else:
+                runtime.stderr += '\n' + errmsg
+            runtime.returncode = 1
+            self.raise_exception(runtime)
+
+        return runtime
+
     def _list_outputs(self):
         outputs = self._outputs().get()
         outputs['BrainExtractionMask'] = os.path.join(os.getcwd(),
@@ -681,6 +737,72 @@ class BrainExtraction(ANTSCommand):
                                                        self.inputs.out_prefix +
                                                        'BrainExtractionBrain.' +
                                                        self.inputs.image_suffix)
+        if isdefined(self.inputs.keep_temporary_files) and self.inputs.keep_temporary_files != 0:
+            outputs['BrainExtractionCSF'] = os.path.join(
+                os.getcwd(),
+                self.inputs.out_prefix + 'BrainExtractionCSF.' + self.inputs.image_suffix
+            )
+            outputs['BrainExtractionGM'] = os.path.join(
+                os.getcwd(),
+                self.inputs.out_prefix + 'BrainExtractionGM.' + self.inputs.image_suffix
+            )
+            outputs['BrainExtractionInitialAffine'] = os.path.join(
+                os.getcwd(),
+                self.inputs.out_prefix +'BrainExtractionInitialAffine.mat'
+            )
+            outputs['BrainExtractionInitialAffineFixed'] = os.path.join(
+                os.getcwd(),
+                self.inputs.out_prefix + 'BrainExtractionInitialAffineFixed.' + self.inputs.image_suffix
+            )
+            outputs['BrainExtractionInitialAffineMoving'] = os.path.join(
+                os.getcwd(),
+                self.inputs.out_prefix + 'BrainExtractionInitialAffineMoving.' + self.inputs.image_suffix
+            )
+            outputs['BrainExtractionLaplacian'] = os.path.join(
+                os.getcwd(),
+                self.inputs.out_prefix + 'BrainExtractionLaplacian.' + self.inputs.image_suffix
+            )
+            outputs['BrainExtractionPrior0GenericAffine'] = os.path.join(
+                os.getcwd(),
+                self.inputs.out_prefix + 'BrainExtractionPrior0GenericAffine.mat'
+            )
+            outputs['BrainExtractionPrior1InverseWarp'] = os.path.join(
+                os.getcwd(),
+                self.inputs.out_prefix + 'BrainExtractionPrior1InverseWarp.' + self.inputs.image_suffix
+            )
+            outputs['BrainExtractionPrior1Warp'] = os.path.join(
+                os.getcwd(),
+                self.inputs.out_prefix + 'BrainExtractionPrior1Warp.' + self.inputs.image_suffix
+            )
+            outputs['BrainExtractionPriorWarped'] = os.path.join(
+                os.getcwd(),
+                self.inputs.out_prefix + 'BrainExtractionPriorWarped.' + self.inputs.image_suffix
+            )
+            outputs['BrainExtractionSegmentation'] = os.path.join(
+                os.getcwd(),
+                self.inputs.out_prefix + 'BrainExtractionSegmentation.' + self.inputs.image_suffix
+            )
+            outputs['BrainExtractionTemplateLaplacian'] = os.path.join(
+                os.getcwd(),
+                self.inputs.out_prefix + 'BrainExtractionTemplateLaplacian.' + self.inputs.image_suffix
+            )
+            outputs['BrainExtractionTmp'] = os.path.join(
+                os.getcwd(),
+                self.inputs.out_prefix + 'BrainExtractionTmp.' + self.inputs.image_suffix
+            )
+            outputs['BrainExtractionWM'] = os.path.join(
+                os.getcwd(),
+                self.inputs.out_prefix + 'BrainExtractionWM.' + self.inputs.image_suffix
+            )
+            outputs['N4Corrected0'] = os.path.join(
+                os.getcwd(),
+                self.inputs.out_prefix + 'N4Corrected0.' + self.inputs.image_suffix
+            )
+            outputs['N4Truncated0'] = os.path.join(
+                os.getcwd(),
+                self.inputs.out_prefix + 'N4Truncated0.' + self.inputs.image_suffix
+            )
+
         return outputs
 
 
@@ -754,7 +876,7 @@ class JointFusion(ANTSCommand):
     ...                                  'segmentation1.nii.gz',
     ...                                  'segmentation1.nii.gz']
     >>> at.inputs.target_image = 'T1.nii'
-    >>> at.cmdline
+    >>> at.cmdline # doctest: +ALLOW_UNICODE
     'jointfusion 3 1 -m Joint[0.1,2] -tg T1.nii -g im1.nii -g im2.nii -g im3.nii -l segmentation0.nii.gz \
 -l segmentation1.nii.gz -l segmentation1.nii.gz fusion_labelimage_output.nii'
 
@@ -763,7 +885,7 @@ class JointFusion(ANTSCommand):
     >>> at.inputs.beta = 1
     >>> at.inputs.patch_radius = [3,2,1]
     >>> at.inputs.search_radius = [1,2,3]
-    >>> at.cmdline
+    >>> at.cmdline # doctest: +ALLOW_UNICODE
     'jointfusion 3 1 -m Joint[0.5,1] -rp 3x2x1 -rs 1x2x3 -tg T1.nii -g im1.nii -g im2.nii -g im3.nii \
 -l segmentation0.nii.gz -l segmentation1.nii.gz -l segmentation1.nii.gz fusion_labelimage_output.nii'
     """
@@ -840,20 +962,20 @@ class DenoiseImage(ANTSCommand):
     >>> denoise = DenoiseImage()
     >>> denoise.inputs.dimension = 3
     >>> denoise.inputs.input_image = 'im1.nii'
-    >>> denoise.cmdline
+    >>> denoise.cmdline # doctest: +ALLOW_UNICODE
     'DenoiseImage -d 3 -i im1.nii -n Gaussian -o im1_noise_corrected.nii -s 1'
 
     >>> denoise_2 = copy.deepcopy(denoise)
     >>> denoise_2.inputs.output_image = 'output_corrected_image.nii.gz'
     >>> denoise_2.inputs.noise_model = 'Rician'
     >>> denoise_2.inputs.shrink_factor = 2
-    >>> denoise_2.cmdline
+    >>> denoise_2.cmdline # doctest: +ALLOW_UNICODE
     'DenoiseImage -d 3 -i im1.nii -n Rician -o output_corrected_image.nii.gz -s 2'
 
     >>> denoise_3 = DenoiseImage()
     >>> denoise_3.inputs.input_image = 'im1.nii'
     >>> denoise_3.inputs.save_noise = True
-    >>> denoise_3.cmdline
+    >>> denoise_3.cmdline # doctest: +ALLOW_UNICODE
     'DenoiseImage -i im1.nii -n Gaussian -o [ im1_noise_corrected.nii, im1_noise.nii ] -s 1'
     """
     input_spec = DenoiseImageInputSpec
@@ -957,12 +1079,12 @@ class AntsJointFusion(ANTSCommand):
     >>> antsjointfusion.inputs.atlas_image = [ ['rc1s1.nii','rc1s2.nii'] ]
     >>> antsjointfusion.inputs.atlas_segmentation_image = ['segmentation0.nii.gz']
     >>> antsjointfusion.inputs.target_image = ['im1.nii']
-    >>> antsjointfusion.cmdline
+    >>> antsjointfusion.cmdline # doctest: +ALLOW_UNICODE
     "antsJointFusion -a 0.1 -g ['rc1s1.nii', 'rc1s2.nii'] -l segmentation0.nii.gz \
 -b 2.0 -o ants_fusion_label_output.nii -s 3x3x3 -t ['im1.nii']"
 
     >>> antsjointfusion.inputs.target_image = [ ['im1.nii', 'im2.nii'] ]
-    >>> antsjointfusion.cmdline
+    >>> antsjointfusion.cmdline # doctest: +ALLOW_UNICODE
     "antsJointFusion -a 0.1 -g ['rc1s1.nii', 'rc1s2.nii'] -l segmentation0.nii.gz \
 -b 2.0 -o ants_fusion_label_output.nii -s 3x3x3 -t ['im1.nii', 'im2.nii']"
 
@@ -970,7 +1092,7 @@ class AntsJointFusion(ANTSCommand):
     ...                                        ['rc2s1.nii','rc2s2.nii'] ]
     >>> antsjointfusion.inputs.atlas_segmentation_image = ['segmentation0.nii.gz',
     ...                                                    'segmentation1.nii.gz']
-    >>> antsjointfusion.cmdline
+    >>> antsjointfusion.cmdline # doctest: +ALLOW_UNICODE
     "antsJointFusion -a 0.1 -g ['rc1s1.nii', 'rc1s2.nii'] -g ['rc2s1.nii', 'rc2s2.nii'] \
 -l segmentation0.nii.gz -l segmentation1.nii.gz -b 2.0 -o ants_fusion_label_output.nii \
 -s 3x3x3 -t ['im1.nii', 'im2.nii']"
@@ -980,7 +1102,7 @@ class AntsJointFusion(ANTSCommand):
     >>> antsjointfusion.inputs.beta = 1.0
     >>> antsjointfusion.inputs.patch_radius = [3,2,1]
     >>> antsjointfusion.inputs.search_radius = [3]
-    >>> antsjointfusion.cmdline
+    >>> antsjointfusion.cmdline # doctest: +ALLOW_UNICODE
     "antsJointFusion -a 0.5 -g ['rc1s1.nii', 'rc1s2.nii'] -g ['rc2s1.nii', 'rc2s2.nii'] \
 -l segmentation0.nii.gz -l segmentation1.nii.gz -b 1.0 -d 3 -o ants_fusion_label_output.nii \
 -p 3x2x1 -s 3 -t ['im1.nii', 'im2.nii']"
@@ -989,7 +1111,7 @@ class AntsJointFusion(ANTSCommand):
     >>> antsjointfusion.inputs.verbose = True
     >>> antsjointfusion.inputs.exclusion_image = ['roi01.nii', 'roi02.nii']
     >>> antsjointfusion.inputs.exclusion_image_label = ['1','2']
-    >>> antsjointfusion.cmdline
+    >>> antsjointfusion.cmdline # doctest: +ALLOW_UNICODE
     "antsJointFusion -a 0.5 -g ['rc1s1.nii', 'rc1s2.nii'] -g ['rc2s1.nii', 'rc2s2.nii'] \
 -l segmentation0.nii.gz -l segmentation1.nii.gz -b 1.0 -d 3 -e 1[roi01.nii] -e 2[roi02.nii] \
 -o ants_fusion_label_output.nii -p 3x2x1 -s mask.nii -t ['im1.nii', 'im2.nii'] -v"
@@ -998,7 +1120,7 @@ class AntsJointFusion(ANTSCommand):
     >>> antsjointfusion.inputs.out_intensity_fusion_name_format = 'ants_joint_fusion_intensity_%d.nii.gz'
     >>> antsjointfusion.inputs.out_label_post_prob_name_format = 'ants_joint_fusion_posterior_%d.nii.gz'
     >>> antsjointfusion.inputs.out_atlas_voting_weight_name_format = 'ants_joint_fusion_voting_weight_%d.nii.gz'
-    >>> antsjointfusion.cmdline
+    >>> antsjointfusion.cmdline # doctest: +ALLOW_UNICODE
     "antsJointFusion -a 0.5 -g ['rc1s1.nii', 'rc1s2.nii'] -g ['rc2s1.nii', 'rc2s2.nii'] \
 -l segmentation0.nii.gz -l segmentation1.nii.gz -b 1.0 -d 3 -e 1[roi01.nii] -e 2[roi02.nii]  \
 -o [ants_fusion_label_output.nii, ants_joint_fusion_intensity_%d.nii.gz, \
@@ -1043,12 +1165,30 @@ ants_joint_fusion_posterior_%d.nii.gz, ants_joint_fusion_voting_weight_%d.nii.gz
             retval = ''
             if not isdefined(self.inputs.out_label_fusion):
                 retval = '-o {0}'.format(self.inputs.out_intensity_fusion_name_format)
+        elif opt == 'atlas_image':
+            atlas_image_cmd = " ".join(
+                ['-g [{0}]'.format(", ".join("'%s'" % fn for fn in ai))
+                                   for ai in self.inputs.atlas_image]
+            )
+            retval = atlas_image_cmd
+        elif opt == 'target_image':
+            target_image_cmd = " ".join(
+                ['-t [{0}]'.format(", ".join("'%s'" % fn for fn in ai))
+                                   for ai in self.inputs.target_image]
+            )
+            retval = target_image_cmd
+        elif opt == 'atlas_segmentation_image':
+            assert len(val) == len(self.inputs.atlas_image), "Number of specified " \
+                "segmentations should be identical to the number of atlas image " \
+                "sets {0}!={1}".format(len(val), len(self.inputs.atlas_image))
+
+            atlas_segmentation_image_cmd = " ".join(
+                ['-l {0}'.format(fn) for fn in self.inputs.atlas_segmentation_image]
+            )
+            retval = atlas_segmentation_image_cmd
         else:
-            if opt == 'atlas_segmentation_image':
-                assert len(val) == len(self.inputs.atlas_image), "Number of specified " \
-                    "segmentations should be identical to the number of atlas image " \
-                    "sets {0}!={1}".format(len(val), len(self.inputs.atlas_image))
-            return super(ANTSCommand, self)._format_arg(opt, spec, val)
+
+            return super(AntsJointFusion, self)._format_arg(opt, spec, val)
         return retval
 
     def _list_outputs(self):
@@ -1067,3 +1207,160 @@ ants_joint_fusion_posterior_%d.nii.gz, ants_joint_fusion_voting_weight_%d.nii.gz
                 self.inputs.out_atlas_voting_weight_name_format)
 
         return outputs
+
+
+class KellyKapowskiInputSpec(ANTSCommandInputSpec):
+    dimension = traits.Enum(3, 2, argstr='--image-dimensionality %d', usedefault=True,
+                            desc='image dimension (2 or 3)')
+
+    segmentation_image = File(exists=True, argstr='--segmentation-image "%s"', mandatory=True,
+                              desc="A segmentation image must be supplied labeling the gray and white matters.\n"
+                                   "Default values = 2 and 3, respectively.",)
+
+    gray_matter_label = traits.Int(2, usedefault=True,
+                                   desc="The label value for the gray matter label in the segmentation_image.")
+
+    white_matter_label = traits.Int(3, usedefault=True,
+                                    desc="The label value for the white matter label in the segmentation_image.")
+
+    gray_matter_prob_image = File(exists=True, argstr='--gray-matter-probability-image "%s"',
+                                  desc="In addition to the segmentation image, a gray matter probability image can be\n"
+                                       "used. If no such image is supplied, one is created using the segmentation image\n"
+                                       "and a variance of 1.0 mm.")
+
+    white_matter_prob_image = File(exists=True, argstr='--white-matter-probability-image "%s"',
+                                   desc="In addition to the segmentation image, a white matter probability image can be\n"
+                                       "used. If no such image is supplied, one is created using the segmentation image\n"
+                                       "and a variance of 1.0 mm.")
+
+    convergence = traits.Str(default="[50,0.001,10]", argstr='--convergence "%s"', usedefault=True,
+                             desc="Convergence is determined by fitting a line to the normalized energy profile of\n"
+                                  "the last N iterations (where N is specified by the window size) and determining\n"
+                                  "the slope which is then compared with the convergence threshold.",)
+
+    thickness_prior_estimate = traits.Float(10, usedefault=True, argstr="--thickness-prior-estimate %f",
+                                            desc="Provides a prior constraint on the final thickness measurement in mm.")
+
+    thickness_prior_image = File(exists=True, argstr='--thickness-prior-image "%s"',
+                                 desc="An image containing spatially varying prior thickness values.")
+
+    gradient_step = traits.Float(0.025, usedefault=True, argstr="--gradient-step %f",
+                                 desc="Gradient step size for the optimization.")
+
+    smoothing_variance = traits.Float(1.0, argstr="--smoothing-variance %f",
+                                      desc="Defines the Gaussian smoothing of the hit and total images.")
+
+    smoothing_velocity_field = traits.Float(1.5, argstr="--smoothing-velocity-field-parameter %f",
+                                            desc="Defines the Gaussian smoothing of the velocity field (default = 1.5).\n"
+                                            "If the b-spline smoothing option is chosen, then this defines the \n"
+                                            "isotropic mesh spacing for the smoothing spline (default = 15).")
+
+    use_bspline_smoothing = traits.Bool(argstr="--use-bspline-smoothing 1",
+                                        desc="Sets the option for B-spline smoothing of the velocity field.")
+
+    number_integration_points = traits.Int(10, argstr="--number-of-integration-points %d",
+                                           desc="Number of compositions of the diffeomorphism per iteration.")
+
+    max_invert_displacement_field_iters = traits.Int(20, argstr="--maximum-number-of-invert-displacement-field-iterations %d",
+                                                     desc="Maximum number of iterations for estimating the invert \n"
+                                                          "displacement field.")
+
+    cortical_thickness = File(argstr='--output "%s"', keep_extension=True,
+                              name_source=["segmentation_image"], name_template='%s_cortical_thickness',
+                              desc='Filename for the cortical thickness.', hash_files=False)
+
+    warped_white_matter = File(name_source=["segmentation_image"], keep_extension=True,
+                               name_template='%s_warped_white_matter',
+                               desc='Filename for the warped white matter file.', hash_files=False)
+
+
+class KellyKapowskiOutputSpec(TraitedSpec):
+    cortical_thickness = File(desc="A thickness map defined in the segmented gray matter.")
+    warped_white_matter = File(desc="A warped white matter image.")
+
+
+class KellyKapowski(ANTSCommand):
+    """ Nipype Interface to ANTs' KellyKapowski, also known as DiReCT.
+
+    DiReCT is a registration based estimate of cortical thickness. It was published
+    in S. R. Das, B. B. Avants, M. Grossman, and J. C. Gee, Registration based
+    cortical thickness measurement, Neuroimage 2009, 45:867--879.
+
+    Examples
+    --------
+    >>> from nipype.interfaces.ants.segmentation import KellyKapowski
+    >>> kk = KellyKapowski()
+    >>> kk.inputs.dimension = 3
+    >>> kk.inputs.segmentation_image = "segmentation0.nii.gz"
+    >>> kk.inputs.convergence = "[45,0.0,10]"
+    >>> kk.inputs.gradient_step = 0.025
+    >>> kk.inputs.smoothing_variance = 1.0
+    >>> kk.inputs.smoothing_velocity_field = 1.5
+    >>> #kk.inputs.use_bspline_smoothing = False
+    >>> kk.inputs.number_integration_points = 10
+    >>> kk.inputs.thickness_prior_estimate = 10
+    >>> kk.cmdline # doctest: +ALLOW_UNICODE
+    u'KellyKapowski --convergence "[45,0.0,10]" \
+--output "[segmentation0_cortical_thickness.nii.gz,segmentation0_warped_white_matter.nii.gz]" \
+--image-dimensionality 3 --gradient-step 0.025000 --number-of-integration-points 10 \
+--segmentation-image "[segmentation0.nii.gz,2,3]" --smoothing-variance 1.000000 \
+--smoothing-velocity-field-parameter 1.500000 --thickness-prior-estimate 10.000000'
+
+    """
+    _cmd = "KellyKapowski"
+    input_spec = KellyKapowskiInputSpec
+    output_spec = KellyKapowskiOutputSpec
+
+    references_ = [{'entry': BibTeX("@book{Das2009867,"
+                                    "author={Sandhitsu R. Das and Brian B. Avants and Murray Grossman and James C. Gee},"
+                                    "title={Registration based cortical thickness measurement.},"
+                                    "journal={NeuroImage},"
+                                    "volume={45},"
+                                    "number={37},"
+                                    "pages={867--879},"
+                                    "year={2009},"
+                                    "issn={1053-8119},"
+                                    "url={http://www.sciencedirect.com/science/article/pii/S1053811908012780},"
+                                    "doi={http://dx.doi.org/10.1016/j.neuroimage.2008.12.016}"
+                                    "}"),
+                    'description': 'The details on the implementation of DiReCT.',
+                    'tags': ['implementation'],
+                    }]
+
+    def _parse_inputs(self, skip=None):
+        if skip is None:
+            skip = []
+        skip += ['warped_white_matter', 'gray_matter_label', 'white_matter_label']
+        return super(KellyKapowski, self)._parse_inputs(skip=skip)
+
+    def _gen_filename(self, name):
+        if name == 'cortical_thickness':
+            output = self.inputs.cortical_thickness
+            if not isdefined(output):
+                _, name, ext = split_filename(self.inputs.segmentation_image)
+                output = name + '_cortical_thickness' + ext
+            return output
+
+        if name == 'warped_white_matter':
+            output = self.inputs.warped_white_matter
+            if not isdefined(output):
+                _, name, ext = split_filename(self.inputs.segmentation_image)
+                output = name + '_warped_white_matter' + ext
+            return output
+
+        return None
+
+    def _format_arg(self, opt, spec, val):
+        if opt == "segmentation_image":
+            newval = '[{0},{1},{2}]'.format(self.inputs.segmentation_image,
+                                            self.inputs.gray_matter_label,
+                                            self.inputs.white_matter_label)
+            return spec.argstr % newval
+
+        if opt == "cortical_thickness":
+            ct = self._gen_filename("cortical_thickness")
+            wm = self._gen_filename("warped_white_matter")
+            newval = '[{},{}]'.format(ct, wm)
+            return spec.argstr % newval
+
+        return super(KellyKapowski, self)._format_arg(opt, spec, val)
