@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
 """
@@ -6,37 +7,34 @@
     >>> filepath = os.path.dirname( os.path.realpath( __file__ ) )
     >>> datadir = os.path.realpath(os.path.join(filepath, '../../testing/data'))
     >>> os.chdir(datadir)
-
 """
+from __future__ import print_function, division, unicode_literals, absolute_import
+from io import open
 
-from __future__ import division
-
-# -*- coding: utf-8 -*-
 import os.path as op
 import nibabel as nb
 import nibabel.trackvis as trk
 import numpy as np
 from nibabel.trackvis import HeaderError
 from nibabel.volumeutils import native_code
+from nibabel.orientations import aff2axcodes
 
-from ..base import (TraitedSpec, BaseInterface, BaseInterfaceInputSpec,
-                    File, isdefined, traits)
+from ... import logging
 from ...utils.filemanip import split_filename
 from ...utils.misc import package_check
 from ...workflows.misc.utils import get_data_dims, get_vox_dims
+from ..base import TraitedSpec, BaseInterface, File, isdefined
 
 import warnings
 have_dipy = True
 try:
     package_check('dipy')
 except Exception as e:
-    False
+    have_dipy = False
 else:
     from dipy.tracking.utils import move_streamlines, affine_from_fsl_mat_file
 
-from nibabel.orientations import aff2axcodes
 
-from ... import logging
 iflogger = logging.getLogger('interface')
 
 
@@ -56,10 +54,11 @@ def read_mrtrix_tracks(in_file, as_generator=True):
 
 
 def read_mrtrix_header(in_file):
-    fileobj = open(in_file, 'r')
+    fileobj = open(in_file, 'rb')
     header = {}
     iflogger.info('Reading header data...')
     for line in fileobj:
+        line = line.decode()
         if line == 'END\n':
             iflogger.info('Reached the end of the header!')
             break
@@ -79,7 +78,7 @@ def read_mrtrix_header(in_file):
 def read_mrtrix_streamlines(in_file, header, as_generator=True):
     offset = header['offset']
     stream_count = header['count']
-    fileobj = open(in_file, 'r')
+    fileobj = open(in_file, 'rb')
     fileobj.seek(offset)
     endianness = native_code
     f4dt = np.dtype(endianness + 'f4')
@@ -139,9 +138,14 @@ def read_mrtrix_streamlines(in_file, header, as_generator=True):
             if n_streams == stream_count:
                 iflogger.info('100% : {n} tracks read'.format(n=n_streams))
                 raise StopIteration
-            if n_streams % int(stream_count / 100) == 0:
-                percent = int(float(n_streams) / float(stream_count) * 100)
-                iflogger.info('{p}% : {n} tracks read'.format(p=percent, n=n_streams))
+            try:
+                if n_streams % int(stream_count / 100) == 0:
+                    percent = int(float(n_streams) / float(stream_count) * 100)
+                    iflogger.info('{p}% : {n} tracks read'.format(p=percent,
+                                                                  n=n_streams))
+            except ZeroDivisionError:
+                iflogger.info('{} stream read out of {}'.format(n_streams,
+                                                                stream_count))
     track_points, nonfinite_list = points_per_track(offset)
     fileobj.seek(offset)
     streamlines = track_gen(track_points)
@@ -167,10 +171,8 @@ class MRTrix2TrackVis(BaseInterface):
     """
     Converts MRtrix (.tck) tract files into TrackVis (.trk) format
     using functions from dipy
-
     Example
     -------
-
     >>> import nipype.interfaces.mrtrix as mrt
     >>> tck2trk = mrt.MRTrix2TrackVis()
     >>> tck2trk.inputs.in_file = 'dwi_CSD_tracked.tck'
@@ -245,7 +247,7 @@ class MRTrix2TrackVis(BaseInterface):
         return outputs
 
     def _gen_filename(self, name):
-        if name is 'out_filename':
+        if name == 'out_filename':
             return self._gen_outfilename()
         else:
             return None
