@@ -4,43 +4,33 @@
 # vi: set ft=python sts=4 ts=4 sw=4 et:
 """Utility routines for workflow graphs
 """
+from __future__ import print_function, division, unicode_literals, absolute_import
+from builtins import str, open, map, next, zip, range
 
-from __future__ import absolute_import
-
+import sys
 from future import standard_library
 standard_library.install_aliases()
+from collections import defaultdict
 
-from builtins import map
-from builtins import next
-from builtins import zip
-from builtins import range
-
-try:
-    import itertools.imap as map
-except ImportError:
-    pass
-
-from collections import OrderedDict
 from copy import deepcopy
 from glob import glob
-from collections import defaultdict
 try:
     from inspect import signature
 except ImportError:
     from funcsigs import signature
+
 import os
 import re
 import pickle
+from functools import reduce
 import numpy as np
 from nipype.utils.misc import package_check
-from functools import reduce
 
 package_check('networkx', '1.3')
 
 import networkx as nx
 
-from ...external.six import string_types
-from ...utils.filemanip import (fname_presuffix, FileNotFoundError,
+from ...utils.filemanip import (fname_presuffix, FileNotFoundError, to_str,
                                 filename_to_list, get_related_files)
 from ...utils.misc import create_function_from_source, str2bool
 from ...interfaces.base import (CommandLine, isdefined, Undefined,
@@ -131,7 +121,7 @@ def format_node(node, format='python', include_config=False):
                                             klass.__class__.__name__)
         comment = '# Node: %s' % node.fullname
         spec = signature(node._interface.__init__)
-        args = [p.name for p in spec.parameters.values()]
+        args = [p.name for p in list(spec.parameters.values())]
         args = args[1:]
         if args:
             filled_args = []
@@ -152,7 +142,10 @@ def format_node(node, format='python', include_config=False):
         lines = [importline, comment, nodedef]
 
         if include_config:
-            lines = [importline, "from collections import OrderedDict",
+            lines = [importline,
+                     "from future import standard_library",
+                     "standard_library.install_aliases()",
+                     "from collections import OrderedDict",
                      comment, nodedef]
             lines.append('%s.config = %s' % (name, node.config))
 
@@ -194,7 +187,7 @@ def modify_paths(object, relative=True, basedir=None):
             out = tuple(out)
     else:
         if isdefined(object):
-            if isinstance(object, string_types) and os.path.isfile(object):
+            if isinstance(object, (str, bytes)) and os.path.isfile(object):
                 if relative:
                     if config.getboolean('execution', 'use_relative_paths'):
                         out = relpath(object, start=basedir)
@@ -277,7 +270,7 @@ def _write_detailed_dot(graph, dotfilename):
         inports = []
         for u, v, d in graph.in_edges_iter(nbunch=n, data=True):
             for cd in d['connect']:
-                if isinstance(cd[0], string_types):
+                if isinstance(cd[0], (str, bytes)):
                     outport = cd[0]
                 else:
                     outport = cd[0][0]
@@ -297,7 +290,7 @@ def _write_detailed_dot(graph, dotfilename):
         outports = []
         for u, v, d in graph.out_edges_iter(nbunch=n, data=True):
             for cd in d['connect']:
-                if isinstance(cd[0], string_types):
+                if isinstance(cd[0], (str, bytes)):
                     outport = cd[0]
                 else:
                     outport = cd[0][0]
@@ -337,6 +330,8 @@ def _get_valid_pathstr(pathstr):
     Removes:  [][ (){}?:<>#!|"';]
     Replaces: ',' -> '.'
     """
+    if not isinstance(pathstr, (str, bytes)):
+        pathstr = to_str(pathstr)
     pathstr = pathstr.replace(os.sep, '..')
     pathstr = re.sub(r'''[][ (){}?:<>#!|"';]''', '', pathstr)
     pathstr = pathstr.replace(',', '.')
@@ -362,7 +357,7 @@ def count_iterables(iterables, synchronize=False):
         op = max
     else:
         op = lambda x, y: x * y
-    return reduce(op, [len(func()) for _, func in iterables.items()])
+    return reduce(op, [len(func()) for _, func in list(iterables.items())])
 
 
 def walk(children, level=0, path=None, usename=True):
@@ -502,7 +497,7 @@ def _merge_graphs(supergraph, nodes, subgraph, nodeid, iterables,
         for edge in supergraph.in_edges_iter(supernodes[nidx]):
                 # make sure edge is not part of subgraph
             if edge[0] not in subgraph.nodes():
-                if n._hierarchy + n._id not in edgeinfo.keys():
+                if n._hierarchy + n._id not in list(edgeinfo.keys()):
                     edgeinfo[n._hierarchy + n._id] = []
                 edgeinfo[n._hierarchy + n._id].append((edge[0],
                                                        supergraph.get_edge_data(*edge)))
@@ -523,9 +518,11 @@ def _merge_graphs(supergraph, nodes, subgraph, nodeid, iterables,
         rootnode = Gc.nodes()[nodeidx]
         paramstr = ''
         for key, val in sorted(params.items()):
-            paramstr = '_'.join((paramstr, _get_valid_pathstr(key),
-                                 _get_valid_pathstr(str(val))))
+            paramstr = '{}_{}_{}'.format(
+                paramstr, _get_valid_pathstr(key), _get_valid_pathstr(val))
             rootnode.set_input(key, val)
+
+        logger.debug('Parameterization: paramstr=%s', paramstr)
         levels = get_levels(Gc)
         for n in Gc.nodes():
             """
@@ -547,7 +544,7 @@ def _merge_graphs(supergraph, nodes, subgraph, nodeid, iterables,
         supergraph.add_nodes_from(Gc.nodes())
         supergraph.add_edges_from(Gc.edges(data=True))
         for node in Gc.nodes():
-            if node._hierarchy + node._id in edgeinfo.keys():
+            if node._hierarchy + node._id in list(edgeinfo.keys()):
                 for info in edgeinfo[node._hierarchy + node._id]:
                     supergraph.add_edges_from([(info[0], node, info[1])])
             node._id += template % i
@@ -651,11 +648,13 @@ def _propagate_internal_output(graph, node, field, connections, portinputs):
         if field in portinputs:
             srcnode, srcport = portinputs[field]
             if isinstance(srcport, tuple) and isinstance(src, tuple):
-                raise ValueError(("Does not support two inline functions "
-                                  "in series (\'%s\'  and \'%s\'). "
-                                  "Please use a Function node") %
-                                 (srcport[1].split("\\n")[0][6:-1],
-                                  src[1].split("\\n")[0][6:-1]))
+                src_func = srcport[1].split("\\n")[0]
+                dst_func = src[1].split("\\n")[0]
+                raise ValueError("Does not support two inline functions "
+                                 "in series ('{}'  and '{}'), found when "
+                                 "connecting {} to {}. Please use a Function "
+                                 "node.".format(src_func, dst_func, srcnode, destnode))
+
             connect = graph.get_edge_data(srcnode, destnode,
                                           default={'connect': []})
             if isinstance(src, tuple):
@@ -712,7 +711,7 @@ def generate_expanded_graph(graph_in):
             in_edges = jedge_dict[jnode] = {}
             edges2remove = []
             for src, dest, data in graph_in.in_edges_iter(jnode, True):
-                in_edges[src._id] = data
+                in_edges[src.itername] = data
                 edges2remove.append((src, dest))
 
             for src, dest in edges2remove:
@@ -724,7 +723,7 @@ def generate_expanded_graph(graph_in):
             # the itersource is a (node name, fields) tuple
             src_name, src_fields = inode.itersource
             # convert a single field to a list
-            if isinstance(src_fields, string_types):
+            if isinstance(src_fields, (str, bytes)):
                 src_fields = [src_fields]
             # find the unique iterable source node in the graph
             try:
@@ -758,7 +757,7 @@ def generate_expanded_graph(graph_in):
             def make_field_func(*pair):
                 return pair[0], lambda: pair[1]
 
-            iterables = dict([make_field_func(*pair) for pair in iter_dict.items()])
+            iterables = dict([make_field_func(*pair) for pair in list(iter_dict.items())])
         else:
             iterables = inode.iterables.copy()
         inode.iterables = None
@@ -797,7 +796,7 @@ def generate_expanded_graph(graph_in):
             expansions = defaultdict(list)
             for node in graph_in.nodes_iter():
                 for src_id, edge_data in list(old_edge_dict.items()):
-                    if node._id.startswith(src_id):
+                    if node.itername.startswith(src_id):
                         expansions[src_id].append(node)
             for in_id, in_nodes in list(expansions.items()):
                 logger.debug("The join node %s input %s was expanded"
@@ -909,7 +908,7 @@ def _standardize_iterables(node):
     if node.synchronize:
         if len(iterables) == 2:
             first, last = iterables
-            if all((isinstance(item, string_types) and item in fields
+            if all((isinstance(item, (str, bytes)) and item in fields
                     for item in first)):
                 iterables = _transpose_iterables(first, last)
 
@@ -1019,42 +1018,48 @@ def export_graph(graph_in, base_dir=None, show=False, use_execgraph=False,
                                suffix='_detailed.dot',
                                use_ext=False,
                                newpath=base_dir)
-    logger.info('Creating detailed dot file: %s' % outfname)
     _write_detailed_dot(graph, outfname)
-    cmd = 'dot -T%s -O %s' % (format, outfname)
-    res = CommandLine(cmd, terminal_output='allatonce').run()
-    if res.runtime.returncode:
-        logger.warn('dot2png: %s', res.runtime.stderr)
+    if format != 'dot':
+        cmd = 'dot -T%s -O %s' % (format, outfname)
+        res = CommandLine(cmd, terminal_output='allatonce').run()
+        if res.runtime.returncode:
+            logger.warn('dot2png: %s', res.runtime.stderr)
     pklgraph = _create_dot_graph(graph, show_connectinfo, simple_form)
-    outfname = fname_presuffix(dotfilename,
-                               suffix='.dot',
-                               use_ext=False,
-                               newpath=base_dir)
-    nx.drawing.nx_pydot.write_dot(pklgraph, outfname)
-    logger.info('Creating dot file: %s' % outfname)
-    cmd = 'dot -T%s -O %s' % (format, outfname)
-    res = CommandLine(cmd, terminal_output='allatonce').run()
-    if res.runtime.returncode:
-        logger.warn('dot2png: %s', res.runtime.stderr)
+    simplefname = fname_presuffix(dotfilename,
+                                  suffix='.dot',
+                                  use_ext=False,
+                                  newpath=base_dir)
+    nx.drawing.nx_pydot.write_dot(pklgraph, simplefname)
+    if format != 'dot':
+        cmd = 'dot -T%s -O %s' % (format, simplefname)
+        res = CommandLine(cmd, terminal_output='allatonce').run()
+        if res.runtime.returncode:
+            logger.warn('dot2png: %s', res.runtime.stderr)
     if show:
         pos = nx.graphviz_layout(pklgraph, prog='dot')
         nx.draw(pklgraph, pos)
         if show_connectinfo:
             nx.draw_networkx_edge_labels(pklgraph, pos)
 
+    if format != 'dot':
+        outfname += '.%s' % format
+        simplefname += '.%s' % format
+    return simplefname if simple_form else outfname
 
-def format_dot(dotfilename, format=None):
+
+def format_dot(dotfilename, format='png'):
     """Dump a directed graph (Linux only; install via `brew` on OSX)"""
-    cmd = 'dot -T%s -O \'%s\'' % (format, dotfilename)
-    try:
-        CommandLine(cmd).run()
-    except IOError as ioe:
-        if "could not be found" in str(ioe):
-            raise IOError("Cannot draw directed graph; executable 'dot' is unavailable")
-        else:
-            raise ioe
-    else:
-        logger.info('Converting dotfile: %s to %s format' % (dotfilename, format))
+    if format != 'dot':
+        cmd = 'dot -T%s -O \'%s\'' % (format, dotfilename)
+        try:
+            CommandLine(cmd).run()
+        except IOError as ioe:
+            if "could not be found" in str(ioe):
+                raise IOError("Cannot draw directed graph; executable 'dot' is unavailable")
+            else:
+                raise ioe
+        dotfilename += '.%s' % format
+    return dotfilename
 
 
 def make_output_dir(outdir):
@@ -1068,12 +1073,12 @@ def make_output_dir(outdir):
     # this odd approach deals with concurrent directory cureation
     try:
         if not os.path.exists(os.path.abspath(outdir)):
-            logger.debug("Creating %s" % outdir)
+            logger.debug("Creating %s", outdir)
             os.makedirs(outdir)
     except OSError:
-            logger.debug("Problem creating %s" % outdir)
+            logger.debug("Problem creating %s", outdir)
             if not os.path.exists(outdir):
-               raise OSError('Could not create %s'%outdir)
+               raise OSError('Could not create %s', outdir)
     return outdir
 
 
@@ -1100,7 +1105,7 @@ def walk_outputs(object):
             if isdefined(val):
                 out.extend(walk_outputs(val))
     else:
-        if isdefined(object) and isinstance(object, string_types):
+        if isdefined(object) and isinstance(object, (str, bytes)):
             if os.path.islink(object) or os.path.isfile(object):
                 out = [(filename, 'f') for filename in get_all_files(object)]
             elif os.path.isdir(object):
