@@ -14,59 +14,41 @@ The `Workflow` class provides core functionality for batch processing.
      os.chdir(datadir)
 
 """
+from __future__ import print_function, division, unicode_literals, absolute_import
+from builtins import range, object, str, bytes, open
 
-from __future__ import absolute_import
-
+# Py2 compat: http://python-future.org/compatible_idioms.html#collections-counter-and-ordereddict
 from future import standard_library
 standard_library.install_aliases()
-from builtins import range
-from builtins import object
 
 from datetime import datetime
-from nipype.utils.misc import flatten, unflatten
-try:
-    from collections import OrderedDict
-except ImportError:
-    from ordereddict import OrderedDict
 
 from copy import deepcopy
 import pickle
-from glob import glob
-import gzip
-import inspect
 import os
 import os.path as op
-import re
 import shutil
-import errno
-import socket
-from shutil import rmtree
 import sys
-from tempfile import mkdtemp
 from warnings import warn
-from hashlib import sha1
 
 import numpy as np
 import networkx as nx
 
-from ...utils.misc import package_check, str2bool
-package_check('networkx', '1.3')
 
 from ... import config, logging
-logger = logging.getLogger('workflow')
+from ...utils.misc import (unflatten, package_check, str2bool,
+                               getsource, create_function_from_source)
 from ...interfaces.base import (traits, InputMultiPath, CommandLine,
                                 Undefined, TraitedSpec, DynamicTraitedSpec,
                                 Bunch, InterfaceResult, md5, Interface,
                                 TraitDictObject, TraitListObject, isdefined)
-from ...utils.misc import (getsource, create_function_from_source,
-                           flatten, unflatten)
+
 from ...utils.filemanip import (save_json, FileNotFoundError,
                                 filename_to_list, list_to_filename,
                                 copyfiles, fnames_presuffix, loadpkl,
                                 split_filename, load_json, savepkl,
                                 write_rst_header, write_rst_dict,
-                                write_rst_list)
-from ...external.six import string_types
+                                write_rst_list, to_str)
 from .utils import (generate_expanded_graph, modify_paths,
                     export_graph, make_output_dir, write_workflow_prov,
                     clean_working_directory, format_dot, topological_sort,
@@ -76,6 +58,8 @@ from .utils import (generate_expanded_graph, modify_paths,
 from .base import EngineBase
 from .nodes import Node, MapNode
 
+package_check('networkx', '1.3')
+logger = logging.getLogger('workflow')
 
 class Workflow(EngineBase):
     """Controls the setup and execution of a pipeline of processes."""
@@ -216,16 +200,21 @@ Trying to connect %s:%s to %s:%s but input '%s' of node '%s' is already
 connected.
 """ % (srcnode, source, destnode, dest, dest, destnode))
                 if not (hasattr(destnode, '_interface') and
-                        '.io' in str(destnode._interface.__class__)):
+                            ('.io' in str(destnode._interface.__class__) or
+                                any(['.io' in str(val) for val in
+                                     destnode._interface.__class__.__bases__]))
+                        ):
                     if not destnode._check_inputs(dest):
                         not_found.append(['in', destnode.name, dest])
                 if not (hasattr(srcnode, '_interface') and
-                        '.io' in str(srcnode._interface.__class__)):
+                            ('.io' in str(srcnode._interface.__class__)
+                             or any(['.io' in str(val) for val in
+                                     srcnode._interface.__class__.__bases__]))):
                     if isinstance(source, tuple):
                         # handles the case that source is specified
                         # with a function
                         sourcename = source[0]
-                    elif isinstance(source, string_types):
+                    elif isinstance(source, (str, bytes)):
                         sourcename = source
                     else:
                         raise Exception(('Unknown source specification in '
@@ -246,7 +235,7 @@ connected.
         # turn functions into strings
         for srcnode, destnode, connects in connection_list:
             for idx, (src, dest) in enumerate(connects):
-                if isinstance(src, tuple) and not isinstance(src[1], string_types):
+                if isinstance(src, tuple) and not isinstance(src[1], (str, bytes)):
                     function_source = getsource(src[1])
                     connects[idx] = ((src[0], function_source, src[2:]), dest)
 
@@ -254,13 +243,13 @@ connected.
         for srcnode, destnode, connects in connection_list:
             edge_data = self._graph.get_edge_data(srcnode, destnode, None)
             if edge_data:
-                logger.debug('(%s, %s): Edge data exists: %s'
-                             % (srcnode, destnode, str(edge_data)))
+                logger.debug('(%s, %s): Edge data exists: %s', srcnode, destnode,
+                             to_str(edge_data))
                 for data in connects:
                     if data not in edge_data['connect']:
                         edge_data['connect'].append(data)
                     if disconnect:
-                        logger.debug('Removing connection: %s' % str(data))
+                        logger.debug('Removing connection: %s', to_str(data))
                         edge_data['connect'].remove(data)
                 if edge_data['connect']:
                     self._graph.add_edges_from([(srcnode,
@@ -268,16 +257,14 @@ connected.
                                                  edge_data)])
                 else:
                     # pass
-                    logger.debug('Removing connection: %s->%s' % (srcnode,
-                                                                  destnode))
+                    logger.debug('Removing connection: %s->%s', srcnode, destnode)
                     self._graph.remove_edges_from([(srcnode, destnode)])
             elif not disconnect:
-                logger.debug('(%s, %s): No edge data' % (srcnode, destnode))
+                logger.debug('(%s, %s): No edge data', srcnode, destnode)
                 self._graph.add_edges_from([(srcnode, destnode,
                                              {'connect': connects})])
             edge_data = self._graph.get_edge_data(srcnode, destnode)
-            logger.debug('(%s, %s): new edge data: %s' % (srcnode, destnode,
-                                                          str(edge_data)))
+            logger.debug('(%s, %s): new edge data: %s', srcnode, destnode, to_str(edge_data))
 
     def disconnect(self, *args):
         """Disconnect nodes
@@ -292,7 +279,7 @@ connected.
                             'of connection tuples (%d args given)' % len(args))
 
         for srcnode, dstnode, conn in connection_list:
-            logger.debug('disconnect(): %s->%s %s' % (srcnode, dstnode, conn))
+            logger.debug('disconnect(): %s->%s %s', srcnode, dstnode, to_str(conn))
             if self in [srcnode, dstnode]:
                 raise IOError(
                     'Workflow connect cannot contain itself as node: src[%s] '
@@ -312,10 +299,10 @@ connected.
                     idx = ed_conns.index(edge)
                     remove.append((edge[0], edge[1]))
 
-            logger.debug('disconnect(): remove list %s' % remove)
+            logger.debug('disconnect(): remove list %s', to_str(remove))
             for el in remove:
                 edge_data['connect'].remove(el)
-                logger.debug('disconnect(): removed connection %s' % str(el))
+                logger.debug('disconnect(): removed connection %s', to_str(el))
 
             if not edge_data['connect']:
                 self._graph.remove_edge(srcnode, dstnode)
@@ -346,8 +333,7 @@ connected.
             return
         for node in newnodes:
             if not issubclass(node.__class__, EngineBase):
-                raise Exception('Node %s must be a subclass of EngineBase' %
-                                str(node))
+                raise Exception('Node %s must be a subclass of EngineBase', node)
         self._check_nodes(newnodes)
         for node in newnodes:
             if node._hierarchy is None:
@@ -439,19 +425,26 @@ connected.
                 base_dir = os.getcwd()
         base_dir = make_output_dir(base_dir)
         if graph2use in ['hierarchical', 'colored']:
+            if self.name[:1].isdigit(): # these graphs break if int
+                raise ValueError('{} graph failed, workflow name cannot begin '
+                                 'with a number'.format(graph2use))
             dotfilename = op.join(base_dir, dotfilename)
             self.write_hierarchical_dotfile(dotfilename=dotfilename,
                                             colored=graph2use == "colored",
                                             simple_form=simple_form)
-            format_dot(dotfilename, format=format)
+            outfname = format_dot(dotfilename, format=format)
         else:
             graph = self._graph
             if graph2use in ['flat', 'exec']:
                 graph = self._create_flat_graph()
             if graph2use == 'exec':
                 graph = generate_expanded_graph(deepcopy(graph))
-            export_graph(graph, base_dir, dotfilename=dotfilename,
-                         format=format, simple_form=simple_form)
+            outfname = export_graph(graph, base_dir, dotfilename=dotfilename,
+                                    format=format, simple_form=simple_form)
+
+        logger.info('Generated workflow graph: %s (graph2use=%s, simple_form=%s).' % (
+            outfname, graph2use, simple_form))
+        return outfname
 
     def write_hierarchical_dotfile(self, dotfilename=None, colored=False,
                                    simple_form=True):
@@ -561,7 +554,7 @@ connected.
         """
         if plugin is None:
             plugin = config.get('execution', 'plugin')
-        if not isinstance(plugin, string_types):
+        if not isinstance(plugin, (str, bytes)):
             runner = plugin
         else:
             name = 'nipype.pipeline.plugins'
@@ -582,7 +575,7 @@ connected.
             crash_dir = self.config['crashdump_dir']
             self.config['execution']['crashdump_dir'] = crash_dir
             del self.config['crashdump_dir']
-        logger.info(str(sorted(self.config)))
+        logger.info('Workflow %s settings: %s', self.name, to_str(sorted(self.config)))
         self._set_needed_outputs(flatgraph)
         execgraph = generate_expanded_graph(deepcopy(flatgraph))
         for index, node in enumerate(execgraph.nodes()):
@@ -799,7 +792,7 @@ connected.
 
     def _set_node_input(self, node, param, source, sourceinfo):
         """Set inputs of a node given the edge connection"""
-        if isinstance(sourceinfo, string_types):
+        if isinstance(sourceinfo, (str, bytes)):
             val = source.get_output(sourceinfo)
         elif isinstance(sourceinfo, tuple):
             if callable(sourceinfo[1]):
@@ -810,7 +803,7 @@ connected.
             newval = dict(val)
         if isinstance(val, TraitListObject):
             newval = val[:]
-        logger.debug('setting node input: %s->%s', param, str(newval))
+        logger.debug('setting node input: %s->%s', param, to_str(newval))
         node.set_input(param, deepcopy(newval))
 
     def _get_all_nodes(self):
@@ -860,29 +853,29 @@ connected.
                              '(DAG)') % self.name)
         nodes = nx.topological_sort(self._graph)
         for node in nodes:
-            logger.debug('processing node: %s' % node)
+            logger.debug('processing node: %s', node)
             if isinstance(node, Workflow):
                 nodes2remove.append(node)
                 # use in_edges instead of in_edges_iter to allow
                 # disconnections to take place properly. otherwise, the
                 # edge dict is modified.
                 for u, _, d in self._graph.in_edges(nbunch=node, data=True):
-                    logger.debug('in: connections-> %s' % str(d['connect']))
+                    logger.debug('in: connections-> %s', to_str(d['connect']))
                     for cd in deepcopy(d['connect']):
-                        logger.debug("in: %s" % str(cd))
+                        logger.debug("in: %s", to_str(cd))
                         dstnode = node._get_parameter_node(cd[1], subtype='in')
                         srcnode = u
                         srcout = cd[0]
                         dstin = cd[1].split('.')[-1]
-                        logger.debug('in edges: %s %s %s %s' %
-                                     (srcnode, srcout, dstnode, dstin))
+                        logger.debug('in edges: %s %s %s %s', srcnode, srcout,
+                                     dstnode, dstin)
                         self.disconnect(u, cd[0], node, cd[1])
                         self.connect(srcnode, srcout, dstnode, dstin)
                 # do not use out_edges_iter for reasons stated in in_edges
                 for _, v, d in self._graph.out_edges(nbunch=node, data=True):
-                    logger.debug('out: connections-> %s' % str(d['connect']))
+                    logger.debug('out: connections-> %s', to_str(d['connect']))
                     for cd in deepcopy(d['connect']):
-                        logger.debug("out: %s" % str(cd))
+                        logger.debug("out: %s", to_str(cd))
                         dstnode = v
                         if isinstance(cd[0], tuple):
                             parameter = cd[0][0]
@@ -897,10 +890,8 @@ connected.
                         else:
                             srcout = parameter.split('.')[-1]
                         dstin = cd[1]
-                        logger.debug('out edges: %s %s %s %s' % (srcnode,
-                                                                 srcout,
-                                                                 dstnode,
-                                                                 dstin))
+                        logger.debug('out edges: %s %s %s %s',
+                                     srcnode, srcout, dstnode, dstin)
                         self.disconnect(node, cd[0], v, cd[1])
                         self.connect(srcnode, srcout, dstnode, dstin)
                 # expand the workflow node
@@ -923,8 +914,13 @@ connected.
             prefix = '  '
         if hierarchy is None:
             hierarchy = []
-        colorset = ['#FFFFC8', '#0000FF', '#B4B4FF', '#E6E6FF', '#FF0000',
-                    '#FFB4B4', '#FFE6E6', '#00A300', '#B4FFB4', '#E6FFE6']
+        colorset = ['#FFFFC8',                       # Y
+                    '#0000FF', '#B4B4FF', '#E6E6FF', # B
+                    '#FF0000', '#FFB4B4', '#FFE6E6', # R
+                    '#00A300', '#B4FFB4', '#E6FFE6', # G
+                    '#0000FF', '#B4B4FF'] # loop B
+        if level > len(colorset) - 2:
+            level = 3 # Loop back to blue
 
         dotlist = ['%slabel="%s";' % (prefix, self.name)]
         for node in nx.topological_sort(self._graph):
@@ -963,8 +959,6 @@ connected.
                                              colored=colored,
                                              simple_form=simple_form, level=level + 3))
                 dotlist.append('}')
-                if level == 6:
-                    level = 2
             else:
                 for subnode in self._graph.successors_iter(node):
                     if node._hierarchy != subnode._hierarchy:
@@ -979,7 +973,7 @@ connected.
                                                            subnode)['connect']:
                             dotlist.append('%s -> %s;' % (nodename,
                                                           subnodename))
-                        logger.debug('connection: ' + dotlist[-1])
+                        logger.debug('connection: %s', dotlist[-1])
         # add between workflow connections
         for u, v, d in self._graph.edges_iter(data=True):
             uname = '.'.join(hierarchy + [u.fullname])
@@ -1003,5 +997,5 @@ connected.
                 if uname1.split('.')[:-1] != vname1.split('.')[:-1]:
                     dotlist.append('%s -> %s;' % (uname1.replace('.', '_'),
                                                   vname1.replace('.', '_')))
-                    logger.debug('cross connection: ' + dotlist[-1])
+                    logger.debug('cross connection: %s', dotlist[-1])
         return ('\n' + prefix).join(dotlist)
